--[[
Name: ComboSounds
Description: Plays sounds when combo-point abilities are ready.
Revision: 
Developed by: Dandelion of Whisperwind (damagemeters at yahoo dot com)
]]

-------------------------------------------------------------------------------
-- Localization (strings are defined in separate files).

local L = AceLibrary("AceLocale-2.2"):new("ComboSounds")

-------------------------------------------------------------------------------
-- Constants.

COMBOSOUNDS_SOUNDSET_SYNTH = L["Synth"];
COMBOSOUNDS_SOUNDSET_ROBOT = L["Robot"];
COMBOSOUNDS_SOUNDSET_CUSTOM = L["Custom"];
ComboSounds_soundSetList = {
	L["Synth"],
	L["Robot"],
	L["Custom"],
};

ComboSounds_soundFiles = {
	COMBO_POINT_GAINED = "Click.wav",
	COMBO_POINTS_GAINED = "Clicks.wav",
	ENERGY_FULL_REPEATED = "Beep.wav",
	ABILITY_READY_COMBOPT = {
		[COMBOSOUNDS_SOUNDSET_SYNTH] = {
			[0] = "S0.wav",
			[1] = "S1.wav", 
			[2] = "S2.wav", 
			[3] = "S3.wav", 
			[4] = "S4.wav", 
			[5] = "S5.wav", 
		},
		[COMBOSOUNDS_SOUNDSET_ROBOT] = {
			[0] = "R0.wav",
			[1] = "R1.wav", 
			[2] = "R2.wav", 
			[3] = "R3.wav", 
			[4] = "R4.wav", 
			[5] = "R5.wav", 
		},
	},
};

-- Ability types.
COMBOSOUNDS_MAIN = 1;
COMBOSOUNDS_FINISHER = 2;

-- Sets.
COMBOSOUNDS_BASE = L["Base"];
COMBOSOUNDS_GROUP = L["Group"];
COMBOSOUNDS_BG = L["Battleground"];

-- Classes
COMBOSOUNDS_CLASS_UNSUPPORTED = 0;
COMBOSOUNDS_CLASS_DRUID = 1;
COMBOSOUNDS_CLASS_ROGUE = 2;

-- Class map: index is the string name returned from wowapi UnitClass("player").
ComboSounds_classMap = {
	DRUID = COMBOSOUNDS_CLASS_DRUID,
	ROGUE = COMBOSOUNDS_CLASS_ROGUE,
};

ComboSounds_abilityDef = {
	[COMBOSOUNDS_CLASS_DRUID] = {
		[COMBOSOUNDS_MAIN] = {
			L["Claw"],
			L["Shred"],
			L["Rake"],
			L["Ravage"],
			L["Pounce"],
			L["Mangle (Cat)"],
		},
		[COMBOSOUNDS_FINISHER] = {
			L["Ferocious Bite"],
			L["Rip"],
			L["Maim"],
		},
	},
	[COMBOSOUNDS_CLASS_ROGUE] = {
		[COMBOSOUNDS_MAIN] = {
			L["Sinister Strike"],
			L["Backstab"],
			L["Gouge"],
			L["Ghostly Strike"],
			L["Hemorrhage"],
			L["Mutilate"],
		},
		[COMBOSOUNDS_FINISHER] = {
			L["Eviscerate"],
			L["Slice and Dice"],
			L["Expose Armor"],
			L["Kidney Shot"],
			L["Rupture"],
			
		},
	};
};

-------------------------------------------------------------------------------
-- Variables.

ComboSounds_playerInCombat = false;
ComboSounds_lastComboCountAnnounced = 0;
ComboSounds_lastEnergyFullComboSoundsTime = -1;
ComboSounds_lastComboAnnounceTime = -1;
ComboSounds_currentComboPoints = 0;
ComboSounds_lastComboSoundTime = -1;
ComboSounds_lastAnnouncedAbilitySet = -1;
ComboSounds_spellIDs = {};

-- Determine our class.
local _, class = UnitClass("player");	
ComboSounds_class = ComboSounds_classMap[class];
if (nil == ComboSounds_class) then
	ComboSounds_class = COMBOSOUNDS_CLASS_UNSUPPORTED;
end

-------------------------------------------------------------------------------
-- ACE 2 definition.

-- ACE instantiation.
ComboSounds = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0") or nil

-------------------------------------------------------------------------------
-- Settings.

ComboSounds_energyFullComboSoundsInterval = 1.0;
ComboSounds_comboReAnnounceInterval = 3.0;

--ComboSounds_profileDefaults = {
--};

ComboSounds_classDefaults = {
	combopt = true;
    energyFull = true;
    soundSet = COMBOSOUNDS_SOUNDSET_SYNTH;
    customSet = {
		[0] = "<MySounds0.wav>",
		[1] = "<MySounds1.wav>",
		[2] = "<MySounds2.wav>",
		[3] = "<MySounds3.wav>",
		[4] = "<MySounds4.wav>",
		[5] = "<MySounds5.wav>",
    };
};
local _, classEnglish = UnitClass("player")
if ("DRUID" == classEnglish) then
	ComboSounds_classDefaults.abilities = {
		[COMBOSOUNDS_BASE] = {
			[COMBOSOUNDS_MAIN] = L["Claw"],
			[COMBOSOUNDS_FINISHER] = L["Ferocious Bite"],
			enabled = true,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
		[COMBOSOUNDS_GROUP] = {
			[COMBOSOUNDS_MAIN] = L["Shred"],
			[COMBOSOUNDS_FINISHER] = L["Rip"],
			enabled = false,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
		[COMBOSOUNDS_BG] = {
			[COMBOSOUNDS_MAIN] = L["Claw"],
			[COMBOSOUNDS_FINISHER] = L["Ferocious Bite"],
			enabled = false,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
	};
else
	ComboSounds_classDefaults.abilities = {
		[COMBOSOUNDS_BASE] = {
			[COMBOSOUNDS_MAIN] = L["Sinister Strike"],
			[COMBOSOUNDS_FINISHER] = L["Eviscerate"],
			enabled = true,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
		[COMBOSOUNDS_GROUP] = {
			[COMBOSOUNDS_MAIN] = L["Backstab"],
			[COMBOSOUNDS_FINISHER] = L["Slice and Dice"],
			enabled = false,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
		[COMBOSOUNDS_BG] = {
			[COMBOSOUNDS_MAIN] = L["Sinister Strike"],
			[COMBOSOUNDS_FINISHER] = L["Eviscerate"],
			enabled = false,
			finisherCheckNum = 5,
			finisherOnly = false,
		},
	};
end

-------------------------------------------------------------------------------
-- ACE event handlers.

function ComboSounds:OnInitialize()
    -- Called when the addon is loaded    
    
    if (COMBOSOUNDS_CLASS_UNSUPPORTED ~= ComboSounds_class) then	
		self:RegisterDB("ComboSoundsDB", "ComboSoundsDBPC")
		--self:RegisterDefaults("profile", ComboSounds_profileDefaults);
		self:RegisterDefaults("class", ComboSounds_classDefaults);
		
		self:SetupOptions();
		self:RegisterChatCommand({"/combosounds", "/cs"}, self.options)
		
		-- Register with waterfall if it's available.
		if waterfall then
			waterfall:Register(
				"ComboSounds",
				"aceOptions", self.options,
				"treeType","TREE",
				"colorR", .8, "colorG", .8, "colorB", .0
			)
			
			if (not waterfall:IsRegistered("ComboSounds")) then
				ComboSounds_Print("WATERFALL NOT REGISTERED");
			end
		else
			ComboSounds_Print("WATERFALL NOT CREATED");
		end
	end
end

-- Called when the addon is enabled
function ComboSounds:OnEnable()
	-- If not available, dont handle any events.  This makes the mod still on but does nothing.
	if (COMBOSOUNDS_CLASS_UNSUPPORTED ~= ComboSounds_class) then
		self:RegisterEvent("PLAYER_REGEN_DISABLED");
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		self:RegisterEvent("LEARNED_SPELL_IN_TAB");
		self:RegisterEvent("PLAYER_ENTERING_WORLD");
		self:ScheduleRepeatingEvent(self.OnUpdate, 0.1, self);
	end
end

function ComboSounds:OnDisable()
    -- Called when the addon is disabled
end

function ComboSounds:OnUpdate()
	local now = GetTime();
	
	---------------------------------------------

	-- Checks to see if we need to play new combo sounds, and does so if we do.
	self:ComboSoundCheck();
	
	---------------------------------------------
	
	-- Handle the "energy full" periodic sound.
	if (self.db.class.energyFull and ComboSounds_playerInCombat and UnitPowerType("player") == 3) then
		local energyFull = (UnitMana("player") == UnitManaMax("player"));
		
		if (	ComboSounds_lastEnergyFullComboSoundsTime > 0 and 
				((now - ComboSounds_lastEnergyFullComboSoundsTime) > ComboSounds_energyFullComboSoundsInterval)) then
			ComboSounds_PlaySound("ENERGY_FULL_REPEATED");
			ComboSounds_lastEnergyFullComboSoundsTime = now;
		elseif (ComboSounds_lastEnergyFullComboSoundsTime < 0 and energyFull) then
			ComboSounds_lastEnergyFullComboSoundsTime = now;
		elseif (not energyFull) then
			ComboSounds_lastEnergyFullComboSoundsTime = -1;
		end
	else
		ComboSounds_lastEnergyFullComboSoundsTime = -1;
	end
end

function ComboSounds:SetupOptions()
	--ComboSounds_PrintD("ComboSounds:SetupOptions called.");

	self.options = { 
		type = "group",
		args = {
			-- Options button.
			options = {
				type = "execute",
				name = L["options"],
				desc = L["Opens options frame..."],
				guiHidden = true,
				func = function() waterfall:Open("ComboSounds"); end
			},
			-- General settings group.
			settings = {
				name = L["General Options"],
				desc = L["General Options"],
				type = "group",
				order = 1,
				cmdHidden = true,
				args = {
					combopt = {
						order = 1,
						type = "toggle",
						name = L["Beep when combo points gained"],
						desc = L["Toggles beeps when combo points are gained."],
						get = function() return self.db.class.combopt; end,
						set = function(v) self.db.class.combopt = v; end,
					},
					energyFull = {
						order = 2,
						type = "toggle",
						name = L["Beep when energy full"],
						desc = L["Toggles beeps when energy bar is full."],
						get = function() return self.db.class.energyFull; end,
						set = function(v) self.db.class.energyFull = v; end,
					},
					soundSet = {
						order = 3,
						type = "text",
						name = L["Sound Set"],
						desc = L["Set of sounds to use."],
						get = function() return self.db.class.soundSet; end,
						set = function(v) self.db.class.soundSet = v; end,
						validate = ComboSounds_soundSetList;
					},
				},
			},
			baseAbilitySet = {
				name = L["Base Abilities"],
				desc = L["BASEMOV_DESC"],
				type = "group",
				order = 2,
				cmdHidden = true,
				args = {
					groupSet = {
						order = 1,
						type = "toggle",
						name = L["Enabled"],
						desc = L["BASETOG_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_BASE].enabled; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BASE].enabled = v; end,
					},
					combo = {
						order = 2,
						type = "text",
						name = L["Combo Ability"],
						desc = L["COMBAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_BASE, COMBOSOUNDS_MAIN); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_BASE, COMBOSOUNDS_MAIN, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_MAIN];
					},
					finisher = {
						order = 3,
						type = "text",
						name = L["Finishing Ability"],
						desc = L["FINAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_BASE, COMBOSOUNDS_FINISHER); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_BASE, COMBOSOUNDS_FINISHER, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_FINISHER];
					},
					finisherComboCount = {
						order = 4,
						type = "range",
						name = L["Finisher Combo Count"],
						desc = L["FCC_DESC"],
						min = 1, max = 5, step = 1,
						get = function() return self.db.class.abilities[COMBOSOUNDS_BASE].finisherCheckNum; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BASE].finisherCheckNum = v; end,
					},
					finisherOnly = {
						order = 5,
						type = "toggle",
						name = L["Finisher Only"],
						desc = L["FINONLY_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_BASE].finisherOnly; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BASE].finisherOnly = v; end,
					},
				},
			},
			groupAbilitySet = {
				name = L["Group Abilities"],
				desc = L["GRPMOV_DESC"],
				type = "group",
				order = 3,
				cmdHidden = true,
				args = {
					groupSet = {
						order = 1,
						type = "toggle",
						name = L["Enabled"],
						desc = L["GRPTOG_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_GROUP].enabled; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_GROUP].enabled = v; end,
					},
					combo = {
						order = 2,
						type = "text",
						name = L["Combo Ability"],
						desc = L["COMBAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_GROUP, COMBOSOUNDS_MAIN); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_GROUP, COMBOSOUNDS_MAIN, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_MAIN];
					},
					finisher = {
						order = 3,
						type = "text",
						name = L["Finishing Ability"],
						desc = L["FINAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_GROUP, COMBOSOUNDS_FINISHER); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_GROUP, COMBOSOUNDS_FINISHER, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_FINISHER];
					},
					finisherComboCount = {
						order = 4,
						type = "range",
						name = L["Finisher Combo Count"],
						desc = L["FCC_DESC"],
						min = 1, max = 5, step = 1,
						get = function() return self.db.class.abilities[COMBOSOUNDS_GROUP].finisherCheckNum; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_GROUP].finisherCheckNum = v; end,
					},
					finisherOnly = {
						order = 5,
						type = "toggle",
						name = L["Finisher Only"],
						desc = L["FINONLY_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_GROUP].finisherOnly; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_GROUP].finisherOnly = v; end,
					},
				},
			},
			bgAbilitySet = {
				name = L["Battleground Abilities"],
				desc = L["BGMOV_DESC"],
				type = "group",
				order = 4,
				cmdHidden = true,
				args = {
					bgSet = {
						order = 1,
						type = "toggle",
						name = L["Enabled"],
						desc = L["BGTOG_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_BG].enabled; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BG].enabled = v; end,
					},
					combo = {
						order = 2,
						type = "text",
						name = L["Combo Ability"],
						desc = L["COMBAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_BG, COMBOSOUNDS_MAIN); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_BG, COMBOSOUNDS_MAIN, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_MAIN];
					},
					finisher = {
						order = 3,
						type = "text",
						name = L["Finishing Ability"],
						desc = L["FINAB_DESC"],
						get = function() return self:GetAbility(COMBOSOUNDS_BG, COMBOSOUNDS_FINISHER); end,
						set = function(v) self:SetAbility(COMBOSOUNDS_BG, COMBOSOUNDS_FINISHER, v); end,
						validate = ComboSounds_abilityDef[ComboSounds_class][COMBOSOUNDS_FINISHER];
					},
					finisherComboCount = {
						order = 4,
						type = "range",
						name = L["Finisher Combo Count"],
						desc = L["FCC_DESC"],
						min = 1, max = 5, step = 1,
						get = function() return self.db.class.abilities[COMBOSOUNDS_BG].finisherCheckNum; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BG].finisherCheckNum = v; end,
					},
					finisherOnly = {
						order = 5,
						type = "toggle",
						name = L["Finisher Only"],
						desc = L["FINONLY_DESC"],
						get = function() return self.db.class.abilities[COMBOSOUNDS_BG].finisherOnly; end,
						set = function(v) self.db.class.abilities[COMBOSOUNDS_BG].finisherOnly = v; end,
					},
				},
			},
			customSet = {
				name = L["Custom Sound Set"],
				desc = L["CUSTSET_DESC"],
				type = "group",
				order = 5,
				cmdHidden = true,
				args = {	
					zeroSound = {
						order = 1,
						type = "text",
						name = L["Zero"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[0]; end,
						set = function(v) self.db.class.customSet[0] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					oneSound = {
						order = 2,
						type = "text",
						name = L["One"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[1]; end,
						set = function(v) self.db.class.customSet[1] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					twoSound = {
						order = 3,
						type = "text",
						name = L["Two"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[2]; end,
						set = function(v) self.db.class.customSet[2] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					threeSound = {
						order = 4,
						type = "text",
						name = L["Three"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[3]; end,
						set = function(v) self.db.class.customSet[3] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					fourSound = {
						order = 5,
						type = "text",
						name = L["Four"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[4]; end,
						set = function(v) self.db.class.customSet[4] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					fiveSound = {
						order = 6,
						type = "text",
						name = L["Five"],
						desc = L["EDITBOX_DESC"],
						usage = L["<filename>"],
						get = function() return self.db.class.customSet[5]; end,
						set = function(v) self.db.class.customSet[5] = v; end,
						--disabled = function() return self.db.class.soundSet ~= COMBOSOUNDS_SOUNDSET_CUSTOM; end,
					},
					--[[
					header = {
						order = 7,
						type = "header",
						name = "Note: After changing the text in any of these boxes you must press enter or it will not save.",
					},
					]]--
				},
			},
		},
	};
end

-------------------------------------------------------------------------------
-- WoW Event handlers.

function ComboSounds:PLAYER_REGEN_DISABLED()
	ComboSounds_playerInCombat = true;
end

function ComboSounds:PLAYER_REGEN_ENABLED()
	ComboSounds_playerInCombat = false;
	ComboSounds_lastComboCountAnnounced = 0;
	ComboSounds_currentComboPoints = 0;
end

function ComboSounds:LEARNED_SPELL_IN_TAB()
	ComboSounds_BuildSpellIDs();
end

function ComboSounds:PLAYER_ENTERING_WORLD()
	ComboSounds_BuildSpellIDs();
end

-------------------------------------------------------------------------------
-- Meat and potatoes.

function ComboSounds:ComboSoundCheck()
	local now = GetTime();

	-- Do nothing if not in combat.
	if (not ComboSounds_playerInCombat) then
		return false;
	end

	-- Update the combo count.
	local comboCount = GetComboPoints();
	if (self.db.class.combopt == true) then
		if (comboCount == ComboSounds_currentComboPoints + 1) then
			ComboSounds_PlaySound("COMBO_POINT_GAINED");
		elseif (comboCount > ComboSounds_currentComboPoints) then
			ComboSounds_PrintD(comboCount.." > "..ComboSounds_currentComboPoints);
			ComboSounds_PlaySound("COMBO_POINTS_GAINED");
		end
	end
	ComboSounds_currentComboPoints = comboCount;

	-- Determine the set to use.
	local abilitySetID;
	if (self.db.class.abilities[COMBOSOUNDS_BG].enabled and (nil ~= GetBattlefieldScore(1))) then
		abilitySetID = COMBOSOUNDS_BG;
	elseif (self.db.class.abilities[COMBOSOUNDS_GROUP].enabled and self:InParty()) then 
		abilitySetID = COMBOSOUNDS_GROUP;
	elseif (self.db.class.abilities[COMBOSOUNDS_BASE].enabled) then
		abilitySetID = COMBOSOUNDS_BASE;
	else
		return;
	end

	-- Determine what spell to check.
	local spellType = COMBOSOUNDS_MAIN;
	if (comboCount >= self.db.class.abilities[abilitySetID].finisherCheckNum) then
		spellType = COMBOSOUNDS_FINISHER;
	elseif (self.db.class.abilities[abilitySetID].finisherOnly) then
		return;
	end
	local spell = self:GetAbility(abilitySetID, spellType);
	if (nil == spell) then
		ComboSounds_PrintD("ERROR: spell not found for "..abilitySetID..", "..spellType);
		return false;
	end
	
	local spellID = ComboSounds_spellIDs[spell];	
	if (spellID == -1) then
		-- We didn't find this spell in the player's spell book, abort.
		ComboSounds_PrintD("spellID not found for "..spell);
		return false;
	end

	-- KLUDGE: For some reason the Bliz api calls are inconsistent in how they want
	-- the Mangle spell defined.  To get the ID, they want "Mangle (Cat)", but to see 
	-- if it is usable and in range they want "Mangle (Cat)()".
	local usableSpell = spell;
	if (spell == L["Mangle (Cat)"]) then
		usableSpell = L["Mangle (Cat)()"];
	end

	-- See if the spell is ready.
	local _, cooldown, _ = GetSpellCooldown(spellID, BOOKTYPE_SPELL);
	local spellReady =	IsUsableSpell(usableSpell) and 
						(1 == IsSpellInRange(usableSpell, "target")) and
						cooldown <= 0;

	-- If it is...
	if (spellReady)	then
		local soundSet;
		if (self.db.class.soundSet == COMBOSOUNDS_SOUNDSET_CUSTOM) then
			soundSet = self.db.class.customSet;
		else
			--ComboSounds_PrintD("sound set = "..self.db.class.soundSet);
			soundSet = ComboSounds_soundFiles["ABILITY_READY_COMBOPT"][self.db.class.soundSet];
		end

		if (comboCount ~= ComboSounds_lastComboCountAnnounced) then
			ComboSounds_PrintD("New combo count = "..comboCount..", "..spell.." ready. (abilitySetID = "..abilitySetID..")");
			ComboSounds_PlaySound(soundSet, comboCount);
			ComboSounds_lastComboCountAnnounced = comboCount;
			ComboSounds_lastComboAnnounceTime = now;
			
			if (ComboSounds_lastAnnouncedAbilitySet ~= abilitySetID) then
				local main = self:GetAbility(abilitySetID, COMBOSOUNDS_MAIN);
				local finisher = self:GetAbility(abilitySetID, COMBOSOUNDS_FINISHER);
				ComboSounds_Print("ComboSounds: now using ability set \'"..abilitySetID.."\': "..main.." / "..finisher..".");
				ComboSounds_lastAnnouncedAbilitySet = abilitySetID;
			end
			
			return true;
		else
			if (now - ComboSounds_lastComboAnnounceTime > ComboSounds_comboReAnnounceInterval) then
				ComboSounds_PrintD("Re-announcing "..ComboSounds_lastComboCountAnnounced);
				ComboSounds_PlaySound(soundSet, ComboSounds_lastComboCountAnnounced);
				ComboSounds_lastComboAnnounceTime = now;
			end
		end
	end
	
	return false;
end

function ComboSounds_PlaySound(soundId, subindex)
	if (nil == soundId) then
		ComboSounds_Print("INTERNAL ERROR: ComboSounds_PlaySound: soundId = nil");
		return;
	end

	local filename;
	if (type(soundId) == "table") then
		filename = soundId;
	else
		filename = ComboSounds_soundFiles[soundId];
		if (nil == filename) then
			ComboSounds_Print("ERROR: ComboSounds_soundFiles["..soundID.."] == nil");
			return;
		end
	end
	
	if (subindex) then
		filename = filename[subindex];
	end
	
	--ComboSounds_PrintD("COMBOSOUNDS! ("..soundId..") : "..filename);
	PlaySoundFile(L["SOUNDFILEPATH"]..filename);
end

-- Based on VFL's BuildCoreSpellDB()
-- /script ComboSounds_BuildSpellIDs();
-- /dm pt ComboSounds_spellIDs
function ComboSounds_BuildSpellIDs()
	
	ComboSounds_spellIDs = {};
	
	--ComboSounds_abilityDef = {
		-- [COMBOSOUNDS_CLASS_DRUID] = {
				--[COMBOSOUNDS_MAIN] = {
					--L["Claw"],
	
	-- Main, Finisher
	for abilityTypeID, abilityTypeList in pairs(ComboSounds_abilityDef[ComboSounds_class]) do
		ComboSounds_PrintD("abilityTypeID = "..abilityTypeID);
		for abilityID, abilityName in pairs(abilityTypeList) do
			ComboSounds_PrintD("abilityID = "..abilityID..", abilityName"..abilityName);	
			local i=1;
			while true do
				local name,q = GetSpellName(i, BOOKTYPE_SPELL);
				if not name then break; end
				--if (not IsPassiveSpell(i, BOOKTYPE_SPELL)) then 	
				if (abilityName == name) then
					ComboSounds_PrintD(abilityName.." = "..i);
					ComboSounds_spellIDs[abilityName] = i;
					break;
				end
				i=i+1;
			end
		
			if (id == -1) then
				-- not found, but not catastrophic
			end	
		end
	end
end

function ComboSounds:GetAbility(set, type)
	return self.db.class.abilities[set][type];	
end

function ComboSounds:SetAbility(set, type, value)
	self.db.class.abilities[set][type] = value;	
end

function ComboSounds:InParty()
	local inParty = false;
	local p = GetNumPartyMembers();
	local r = GetNumRaidMembers();

	if ((p + r) > 0) then
		inParty = true;
	end

	return inParty;
end

-------------------------------------------------------------------------------
-- Handy funcs.

function ComboSounds_Print(msg, color, bSecondChatWindow)
	local r = 1.00;
	local g = 1.00;
	local b = 1.00;

	if (color) then
		r = color.r;
		g = color.g;
		b = color.b;
	end

	local frame = DEFAULT_CHAT_FRAME;
	if (bSecondChatWindow) then
		frame = ChatFrame2;
	end

	if (frame) then
		frame:AddMessage(msg,r,g,b);
	end
end

function ComboSounds_PrintD(msg, color, bSecondChatWindowArg)
	--[[ Comment out for release builds.
	local bSecondChatWindow = true;
	if (bSecondChatWindowArg) then
		bSecondChatWindow = bSecondChatWindowArg;
	end
	msg = string.format("[%.3f]: "..msg, GetTime());
	ComboSounds_Print(msg, color, bSecondChatWindow);
	]]--
end

